/*
 * Small AJAX framework
 */
function ajax_request_callback(req)
{
    if (req.readyState == 4 && req.status == 200) {
	var data = json_parse(req.responseText);

	if (typeof data.error == 'string') {
	    if (typeof data.errormsg == 'string')
		error(data.errormsg);
	    else
		error(data.error);
	    alert(data.error);
	} else if (req.oncomplete)
	    req.oncomplete(data);
    }
}

function ajax_request(func,params,oncomplete)
{
    var req = new XMLHttpRequest();
    if (!req) return;

    var params = 'func=' + func + '&' + params;
    req.open ("POST", 'ajax/ajax.php');
    req.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
    req.setRequestHeader("Content-length", params.length);
    req.setRequestHeader("Connection", "close");
    req.onreadystatechange = function() { ajax_request_callback(req); }
    if (typeof oncomplete == 'function')
	req.oncomplete = oncomplete;
    req.send(params);
    info('');
}

function ajax_form_submit(func,form,oncomplete)
{
    var boundary = "---------------------------" + (new Date).getTime();
    var CRLF  = "\r\n";
    var req = new XMLHttpRequest();
    if (!req) return;

    var parts = new Array();

    if (func && func.length) {
        var part = 'Content-Disposition: form-data; ';
        part += 'name="func"' + CRLF + CRLF;
        part += func + CRLF;
        parts.push(part);
    }

    for (var i=0; i < form.childNodes.length; i++) {
	var part = '';
	var element = form.childNodes[i];
	var fieldName = element.name;

	if (element.nodeName.toLowerCase() == 'input' &&
	    element.type.toLowerCase() == 'file') {

	    try {
		var binary = element.files[0].getAsBinary();
	    } catch(err) {
		error('Problem beim Lesen der Datei');
		alert("Problem beim Lesen der Datei\n" +
		      element.files[0].fileName + "\n" +
		      "Eventuell stimmt das Encoding nicht\n" +
		      "Die Daten wurden nicht gespeichert");
		return;
	    }

	    part = 'Content-Disposition: form-data; ';
	    part += 'name="' + fieldName + '"; ';
	    part += 'filename="'+ element.files[0].fileName + '"' + CRLF;
	    part += "Content-Type: application/octet-stream" + CRLF + CRLF;
	    part += binary + CRLF;
	} else if (element.nodeName.toLowerCase() == 'input' ||
		   element.nodeName.toLowerCase() == 'textarea' ||
		   element.nodeName.toLowerCase() == 'select') {
	    var value = '';
	    if (element.type.toLowerCase() == 'checkbox') {
		if (element.checked) value = 'on';
		else value = 'off';
	    } else {
		value = element.value;
	    }

	    part = 'Content-Disposition: form-data; ';
	    part += 'name="' + element.name + '"' + CRLF + CRLF;
	    part += value + CRLF;
	} else
	    continue;
        parts.push(part);
    }

    var params = "--" + boundary + CRLF;
    params += parts.join("--" + boundary + CRLF);
    params += "--" + boundary + "--" + CRLF;

    req.open ("POST", 'ajax/ajax.php');
    req.setRequestHeader("Content-Type", "multipart/form-data; boundary=" + boundary);
    req.setRequestHeader("Connection", "close");
    req.onreadystatechange = function() { ajax_request_callback(req); }
    if (typeof oncomplete == 'function')
	req.oncomplete = oncomplete;
    req.sendAsBinary(params);

    info('');
    form_clear(form);
}

function info(msg)
{
    var status = document.getElementById('status');
    if (!status) return;

    status.innerHTML = msg;
    status.className = 'status_ok';
}

function error(msg)
{
    var status = document.getElementById('status');
    if (!status) return;

    status.innerHTML = msg;
    status.className = 'status_error';
}

function button_enable(id)
{
    var button = document.getElementById(id);
    if (!button) return;

    button.className = '';
    button.disabled = false;
}

function button_disable(id)
{
    var button = document.getElementById(id);
    if (!button) return;

    button.className = 'disabled';
    button.disabled = true;
}

function set_value(id, value)
{
    var obj = document.getElementById(id);
    if (!obj) return;

    if (obj.nodeName.toLowerCase() == 'input') {
	if (obj.type.toLowerCase() == 'checkbox') {
	    if (value) obj.checked = true;
	    else obj.checked = false;
	} else if (obj.type.toLowerCase() == 'file')
	    return;
	else
	    obj.value = value;
    } else if (obj.nodeName.toLowerCase() == 'textarea')
	obj.value = value;
    else if (obj.nodeName.toLowerCase() == 'span')
	obj.innerHTML = value;
    else if (obj.nodeName.toLowerCase() == 'select') {
	for (var i=0; i < obj.options.length; i++)
	    if (obj.options[i].value == value)
		obj.selectedIndex = i;
    } else
	obj.innerHTML = value;
}

function setvar(obj, name, callback, status)
{
    if (!obj.options[obj.selectedIndex].value.length)
	value = -1;
    else
	value = obj.options[obj.selectedIndex].value;

    var source = document.getElementById('source');

    if (!source) return false;

    var parms = 'source=' + source.innerHTML + '&name=' + name + '&value=' + value;

    ajax_request('setvar', parms, callback);

    if (typeof status == 'function')
	status(obj,value,obj.options[obj.selectedIndex].innerHTML);
}

function get_info(name, values, callback)
{
    var source = document.getElementById('source');

    if (!source) return false;

    var parms = 'source=' + source.innerHTML + '&name=' + name;
    for (key in values)
	parms += '&' + key + '=' + values[key];

    ajax_request('info', parms, callback);
}

/*
 * Form functions
 */
function form_clear(form)
{
    if (!form) return;

    for (var i=0; i < form.childNodes.length; i++)
	if (form.childNodes[i].nodeName.toLowerCase() == 'input'
	    && (form.childNodes[i].type.toLowerCase() == 'password' ||
		form.childNodes[i].type.toLowerCase() == 'file'))
	    form.childNodes[i].value = '';

    var id = document.getElementById('edit_id');
    if (id) id.value = '';
}

function form_init()
{
    var form = document.getElementById('form_edit');

    if (!form) return;

    form_clear(form);

    var elem = form.findFirstElement();
    if (elem) elem.activate();
}

function form_elem_error(elem,text)
{
    elem.style.borderColor = 'red';
    elem.style.borderWidth = '1px';
    elem.style.borderStyle = 'solid';
    error(text);
}

function form_elem_ok(elem)
{
    elem.style.borderColor = '';
    elem.style.borderWidth = '';
    elem.style.borderStyle = '';
}

function form_reset_errors()
{
   for (name in form_check) {
	var elem = document.getElementById('edit_' + name);
	form_elem_ok(elem);
    }
}

var form_check_regexp = {
    number: /^\d+$/,
    decimal: /^\d*([,\.]\d+)?$/,
    date: /^(\d\d?\.\d\d?\.[1-9]\d\d\d)|([1-9]\d\d\d-\d\d?-\d\d?)$/
};

function form_elem_check(elem,check)
{
    var checks = check.split(',');

    for (var i=0; i < checks.length; i++) {
	var tagName = elem.tagName.toLowerCase();
	var label = document.getElementById('label_' + elem.name);
	if (checks[i] == 'required') {
	    if ((tagName == 'input' ||
		 tagName == 'textarea') &&
		elem.value.length == 0)
		return label.innerHTML + ' muss ausgefüllt werden';

	    if (tagName == 'select' && elem.value.length == 0)
		return label.innerHTML + ' muss ausgewählt werden';
	} else if (form_check_regexp[checks[i]]) {
	    if (elem.value.length && !form_check_regexp[checks[i]].test(elem.value))
		return 'Ungültiger Wert für ' + label.innerHTML;
	}
    }
    form_elem_ok(elem);
    return '';
}

function form_elem_onblur(e)
{
    if (form_check[e.originalTarget.name]) {
	var error = form_elem_check(e.originalTarget, form_check[e.originalTarget.name]);
	if (error.length)
	    form_elem_error(e.originalTarget, error);
    }
}

function form_checks(form)
{
    var errors = '';
    for (name in form_check) {
	var elem = document.getElementById('edit_' + name);
	var error = form_elem_check(elem, form_check[name]);
	if (error.length) {
	    form_elem_error(elem, error);
	    errors += '. ' + error + "\n";
	}
    }

    if (errors.length) {
	alert("Fehler im Formular:\n" + errors);
	return false;
    }

    return true;
}

function select_update(id, options, empty)
{
    var obj = document.getElementById(id);
    if (!obj) return;

    if (typeof empty == 'undefined') empty = 0;

    obj.options.length = empty;

    for (var i=0; i < options.length; i++)
      obj.options[empty+i] = new Option(options[i].text,options[i].id,false,false);
}

var pre_save = false;
var pre_insert = false;
var post_save = function() { grid_update(grid); }
var post_delete = function() { grid_update(grid); }

function save_callback(data)
{
    info('Datensatz gespeichert');

    if (typeof post_save == 'function')
	post_save();

    var form = document.getElementById('form_edit');

    if (!form) return;

    form_clear(form);
    Form.focusFirstElement(form);
}

function delete_callback(data)
{
    info('Datensatz gelöscht');

    if (typeof post_delete == 'function')
	post_delete();
}

function form_save(obj)
{
    var id = document.getElementById('edit_id');

    if (!id.value.length)
	return form_insert(obj);

    if (!form_checks(obj.form))
	return false;

    info('');

    if (typeof pre_save == 'function')
	if (!pre_save())
	    return false;


    ajax_request('save', Form.serialize(obj.form), save_callback);
    return false;
}

function form_insert(obj)
{
    if (!form_checks(obj.form))
	return false;

    info('');

    if (typeof pre_insert == 'function')
	if (!pre_insert())
	    return false;

    ajax_form_submit('insert', obj.form, save_callback);
    return false;
}

function form_delete(obj)
{
    var id = document.getElementById('edit_id');

    if (!id.value.length) return false;

    var source = document.getElementById('edit_source');
    info('');
    var params = 'id='+id.value + '&source='+source.value;
    ajax_request('delete', params, delete_callback);
    return false;
}

/*
 * Table functions
 */
function details_callback(data)
{
    for (var id in data)
	set_value('detail_'+id, data[id]);
}

function fetch_callback(data)
{
    for (var id in data)
	set_value('edit_'+id, data[id]);

    var status = document.getElementById('form_status');
    status.innerHTML = 'Geändert: ' + data.sys_edit + ' von ' + data.sys_user;
}

/*
 * Rico functions
 */
function gridDrillDown(e)
{
    if (e.originalTarget && e.originalTarget.target && e.originalTarget.target == '_top')
	return;

    if (e.originalTarget && 
	e.originalTarget.tagName.toLowerCase() !== 'span' &&
	(!e.originalTarget.className || (e.originalTarget.className.split('_'))[0] !== 'ricoLG'))
	return;

    var id = 0; // Column 0 contains ID
    var row = grid.edit.drillDown(e,0,0);
    var cell = grid.columns[id].cell(row);
    if (!cell) return;
    var value = cell.innerHTML;

    if (!value.length || value == '&nbsp;') return;

    if (document.getElementById('details')) {
	var params = 'source=' + grid.tableId.substr(5) + '&id=' + value;
	ajax_request('details', params, details_callback);
    }

    if (document.getElementById('form_edit')) {
	var params = 'source=' + grid.tableId.substr(5) + '&id=' + value;
	ajax_request('fetch', params, fetch_callback);
	form_reset_errors();
    }

    if (typeof second != 'undefined') {
	second.buffer.options.requestParameters = ['second_id=' + value];
	grid_update(second);
    }

    if (typeof drilldown == 'function')
	drilldown(e,value);
}

function gridOnScroll(grid, offset)
{
    var max = Math.min(offset+grid.pageSize, grid.buffer.totalRows);
    var info = document.getElementById('info_' + grid.tableId);
    info.innerHTML = 'Datensatz ' + (offset+1) + ' - ' + max + ' von ' + grid.buffer.totalRows;
}

/* Update an existing grid
 *
 * grid is a live grid
 * filter is the array index of the $table_filters array in mskdef
 * value is the value applied to the filter
 */
function grid_update(grid, filter, value)
{
    if (grid === undefined)
	return;

    if (filter !== undefined && filter !== false)
		grid.buffer.options.requestParameters = ['w'+filter+'=' + value];
    grid.buffer.clear();
    grid.buffer.setTotalRows(0);
    grid.buffer.foundRowCount = false;
    grid.cancelMenu();
    grid.ClearSelection();
    grid.setImages();
    if (grid.bookmark) grid.bookmark.innerHTML="&nbsp;";
    grid.clearRows();
    grid.buffer.fetch(-1);
}

function grid_update_filters(grid)
{
    var todo = false;
    for (var c=0; c < grid.headerColCnt; c++) {
	var fmt = grid.columns[c].format;
	if (typeof fmt.filterUI != 'string') continue;
	if (fmt.filterUI != 's') continue;

	if (grid.columns[c]._getdesc) continue;

	$(grid.filterId(c)).options.length = 1;

	var options = {};
	Object.extend(options, grid.buffer.ajaxOptions);
	var colnum = typeof(fmt.filterCol)=='number' ? fmt.filterCol : c;

	options.parameters = 'id='+grid.tableId+'&distinct='+colnum;
	options.onComplete = grid.filterValuesUpdate.bind(grid,c);
	new Ajax.Request(grid.buffer.dataSource, options);
    }
}

var grid_column_edit = new Array();

function grid_cell_save(id,col,value)
{
    var source = document.getElementById('source');

    if (!source) return false;

    var parms = 'source=' + source.innerHTML;
    parms += '&callback=cellsave';
    parms += '&id=' + id;
    parms += '&column=' + col;
    parms += '&value=' + value;

    ajax_request('function', parms, false);
}

function grid_cell_value()
{
    this.grid.menu.cancelmenu();

    var bufRow = this.row;
    if (this.grid.lastRowPos > 0) bufRow += this.grid.lastRowPos;

    this.grid.buffer.setValue(bufRow, this.col, this.visible);

    var cell = this.grid.cell(this.row,this.col);
    cell.innerHTML = this.visible;

    grid_cell_save(this.grid.buffer.getCell(bufRow,0), this.col, this.value);
}

function grid_dataMenuHandler(grid,row,col,onBlankRow)
{
    var default_menu = true;

    if (grid_column_edit[col] &&
	typeof grid_column_edit[col].default_menu !== 'undefined' &&
	grid_column_edit[col].default_menu == false)
	default_menu = false;

    if (default_menu && typeof grid.menu.options.dataMenuHandlerOriginal == 'function') {
	grid.menu.options.dataMenuHandlerOriginal(grid,row,col,onBlankRow);
	grid.menu.div.style.width = '17em';
    }

    if (grid_column_edit[col]) {
	var submenu;
	if (default_menu) {
	    var submenu = new Rico.Menu(grid_column_edit[col].width);
	    submenu.createDiv();
	} else
	    submenu = grid.menu;

	for (i=0; i < grid_column_edit[col].values.length; i++)
	    submenu.addMenuItem(grid_column_edit[col].values[i][0],
				grid_cell_value.bind({grid: grid, row: row, col: col,
					    visible: grid_column_edit[col].values[i][1],
					    value: grid_column_edit[col].values[i][2]}));

	if (!default_menu && typeof grid_column_edit[col].width !== 'undefined')
	    grid.menu.div.style.width = grid_column_edit[col].width;

	if (default_menu)
	    grid.menu.addSubMenuItem('Wert setzen', submenu, true);
    }

    if (default_menu)
	return true;
    else
	return false;
}

var calendars = new Array();
function calendar_callback(value)
{
    this.input.value = value;
}

function calendar(name,event)
{
    var input = document.getElementById(name);
    if (!input) return;

    if (calendars[name] == undefined) {
        calendars[name] = new Rico.CalendarControl('calendar_'+name,
						   {startAt: 1,
						    dateFmt: 'dd.mm.yyyy',
						    showWeekNumber: 1});
        calendars[name].atLoad();
        calendars[name].returnValue = calendar_callback;
        RicoUtil.positionCtlOverIcon(calendars[name].container,input);
        calendars[name].open(input.value);
	calendars[name].input = input;
    } else {
        if (Element.visible(calendars[name].container))
            calendars[name].close();
        else
            calendars[name].open(input.value);
    }
    Event.stop(event);
}
